﻿using System;
using Microsoft.Xna.Framework;
using Terraria;
using Terraria.ID;
using Terraria.ModLoader;

namespace SlimeBiome.Textures.NPCs
{
	public class FlutterSlime : ModNPC
	{
		public override void SetStaticDefaults()
		{
			DisplayName.SetDefault("Flutter Slime");
			Main.npcFrameCount[npc.type] = 6;
		}

		public override void SetDefaults()
		{
			npc.width = 32;
			npc.height = 32;
			npc.aiStyle = -1;
			npc.damage = 50;
			npc.defense = 2;
			npc.lifeMax = 150;
			npc.HitSound = SoundID.NPCHit1;
			npc.DeathSound = SoundID.NPCDeath1;
			npc.value = 25f;
			npc.knockBackResist = 0.2f;
		}

		public override float SpawnChance(NPCSpawnInfo spawnInfo)
		{
			return spawnInfo.player.GetModPlayer<SlimeBiomePlayer>().ZoneSlimeBiome ? 50f : 0f;
		}

		const int AI_State_Slot = 0;
		const int AI_Timer_Slot = 1;
		const int AI_Flutter_Time_Slot = 2;
		//Changed by Quethed
		const int AI_Timer_Slot2 = 3;
		//End of change

		const int Local_AI_Unused_Slot_0 = 0;
		const int Local_AI_Unused_Slot_1 = 1;
		const int Local_AI_Unused_Slot_2 = 2;
		const int Local_AI_Unused_Slot_3 = 3;

		const int State_Asleep = 0;
		const int State_Notice = 1;
		const int State_Jump = 2;
		const int State_Hover = 3;
		const int State_Fall = 4;

		// This is a property (https://msdn.microsoft.com/en-us/library/x9fsa0sw.aspx)
		public float AI_State
		{
			get { return npc.ai[AI_State_Slot]; }
			set { npc.ai[AI_State_Slot] = value; }
		}

		public float AI_Timer
		{
			get { return npc.ai[AI_Timer_Slot]; }
			set { npc.ai[AI_Timer_Slot] = value; }
		}

		public float AI_FlutterTime
		{
			get { return npc.ai[AI_Flutter_Time_Slot]; }
			set { npc.ai[AI_Flutter_Time_Slot] = value; }
		}
		//Added by Quethed
		public float AI_Timer2
		{
			get { return npc.ai[AI_Timer_Slot2]; }
			set { npc.ai[AI_Timer_Slot2] = value; }
		}
		//End of addition

		public override void AI()
		{
			if (AI_State == State_Asleep)
			{
				npc.TargetClosest(true);
				if (npc.HasValidTarget && Main.player[npc.target].Distance(npc.Center) < 500f)
				{
					AI_State = State_Notice;
					AI_Timer = 0;
				}
			}
			else if (AI_State == State_Notice)
			{
				if (Main.player[npc.target].Distance(npc.Center) < 250f)
				{
					AI_Timer++;
					if (AI_Timer >= 20)
					{
						AI_State = State_Jump;
						AI_Timer = 0;
					}
				}
				else
				{
					npc.TargetClosest(true);
					if (!npc.HasValidTarget || Main.player[npc.target].Distance(npc.Center) > 500f)
					{
						AI_State = State_Asleep;
						AI_Timer = 0;
					}
				}
			}
			else if (AI_State == State_Jump)
			{
				AI_Timer++;
				if (AI_Timer == 1)
				{ 
					npc.velocity = new Vector2(npc.direction * 2, -10f);
				}
				else if (AI_Timer > 40)
				{
					AI_State = State_Hover;
					AI_Timer = 0;
				}
			}
			else if (AI_State == State_Hover)
			{
				AI_Timer += 1;
				if (AI_Timer == 1 && Main.netMode != 1)
				{
					AI_FlutterTime = Main.rand.NextBool() ? 100 : 50;
					npc.netUpdate = true;
				}
				npc.velocity += new Vector2(0, -.35f);
				if (Math.Abs(npc.velocity.X) < 2)
				{
					npc.velocity += new Vector2(npc.direction * .05f, 0);
				}
				//Added by Quethed
				Player player = Main.player[npc.target];
				npc.TargetClosest(true);
					AI_Timer2++;
				if (AI_Timer2 > 15)
				{	
					int proj = Projectile.NewProjectile(npc.Center.X, npc.Center.Y, 0f, 0f, mod.ProjectileType("Gel"), npc.damage / 2, 40f, Main.myPlayer, player.Center.X, player.Center.Y);
					AI_Timer2 = 0;
				}
				//End of addition
				if (AI_Timer > AI_FlutterTime)
				{
					AI_State = State_Fall;
					AI_Timer = 0;
				}
			}
			else if (AI_State == State_Fall)
			{
				if (npc.velocity.Y == 0)
				{
					npc.velocity.X = 0;
					AI_State = State_Asleep;
					AI_Timer = 0;
				}
			}
		}

		const int Frame_Asleep = 0;
		const int Frame_Notice = 1;
		const int Frame_Falling = 2;
		const int Frame_Flutter_1 = 3;
		const int Frame_Flutter_2 = 4;
		const int Frame_Flutter_3 = 5;

		public override void FindFrame(int frameHeight)
		{
			npc.spriteDirection = npc.direction;

			if (AI_State == State_Asleep)
			{
				npc.frame.Y = Frame_Asleep * frameHeight;
			}
			else if (AI_State == State_Notice)
			{
				if (AI_Timer < 10)
				{
					npc.frame.Y = Frame_Notice * frameHeight;
				}
				else
				{
					npc.frame.Y = Frame_Asleep * frameHeight;
				}
			}
			else if (AI_State == State_Jump)
			{
				npc.frame.Y = Frame_Falling * frameHeight;
			}
			else if (AI_State == State_Hover)
			{
				npc.frameCounter++;
				if (npc.frameCounter < 10)
				{
					npc.frame.Y = Frame_Flutter_1 * frameHeight;
				}
				else if (npc.frameCounter < 20)
				{
					npc.frame.Y = Frame_Flutter_2 * frameHeight;
				}
				else if (npc.frameCounter < 30)
				{
					npc.frame.Y = Frame_Flutter_3 * frameHeight;
				}
				else
				{
					npc.frameCounter = 0;
				}
			}
			else if (AI_State == State_Fall)
			{
				npc.frame.Y = Frame_Falling * frameHeight;
			}
		}

		public override void NPCLoot()
		{
			if (Main.rand.Next(1) == 0)
			{
				Item.NewItem((int)npc.position.X, (int)npc.position.Y, npc.width, npc.height, ItemID.Gel, Main.rand.Next(1, 6));
			}
			if (Main.rand.Next(100) == 0)
			{
				Item.NewItem((int)npc.position.X, (int)npc.position.Y, npc.width, npc.height, ItemID.SlimeStaff);
			}
			if (Main.rand.Next(100) == 0)
			{
				Item.NewItem((int)npc.position.X, (int)npc.position.Y, npc.width, npc.height, mod.ItemType("GelBlaster"));
			}
		}
	}
}
